""" Views for the Curate Data Structure REST API
"""
import logging

from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_curate_app.components.curate_data_structure import (
    api as data_structure_api,
)
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.rest.curate_data_structure.admin_serializers import (
    CurateDataStructureAdminSerializer,
)
from core_curate_app.rest.curate_data_structure.serializers import (
    CurateDataStructureSerializer,
)

logger = logging.getLogger(__name__)


class AdminCurateDataStructureList(APIView):
    """List all Curate Data Structure, or create a new one."""

    permission_classes = (IsAdminUser,)
    serializer = CurateDataStructureAdminSerializer

    def get(self, request):
        """Get all user Curate Data Structure

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of curate data structure
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)

        try:
            # Get object
            object_list = CurateDataStructure.get_all()

            # Serialize object
            serializer = self.serializer(object_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def post(self, request):
        """Create a Curate Data Structure

        Parameters:

            {
                "user": "1",
                "name": "name",
                "form_string": "<xml></xml>",
                "template": "5eb1cc6d53d26cbd4085c722",
                "data_structure_element_root": "5eb36cc1b72a6298744d746a",
                "data": "5eb36ca0b72a6298744d724b"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created data
            - code: 400
              content: Validation error
            - code: 404
              content: Template was not found
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)

        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Object not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class CurateDataStructureList(APIView):
    """List Curate Data Structure by user, create a new one."""

    permission_classes = (IsAuthenticated,)
    serializer = CurateDataStructureSerializer

    def get(self, request):
        """Get all user Curate Data Structure

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of curate data structure
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            object_list = data_structure_api.get_all_by_user(request.user)

            # Serialize object
            serializer = self.serializer(object_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def post(self, request):
        """Create a Curate Data Structure

        Parameters:

            {
                "name": "name",
                "form_string": "<xml></xml>",
                "template": "5eb1cc6d53d26cbd4085c722",
                "data_structure_element_root": "5eb36cc1b72a6298744d746a",
                "data": "5eb36ca0b72a6298744d724b"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created data
            - code: 400
              content: Validation error
            - code: 404
              content: Template was not found
            - code: 500
              content: Internal server error
        """

        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Object not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class CurateDataStructureDetail(APIView):
    """Retrieve, update or delete a Curate Data Structure"""

    permission_classes = (IsAuthenticated,)
    serializer = CurateDataStructureSerializer

    def get_object(self, request, pk):
        """Get Curate Data Structure from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Data
        """
        try:
            return data_structure_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Retrieve a Curate Data Structure

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content:  Curate Data Structure
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)

            # Serialize object
            serializer = self.serializer(data_structure_object)

            # Return response
            return Response(serializer.data)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def delete(self, request, pk):
        """Delete a Curate Data Structure

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)

            # delete object
            data_structure_api.delete(data_structure_object, request.user)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def patch(self, request, pk):
        """Update a  Curate Data Structure

         Parameters:

            {
                "name": "new_name",
                "form_string": "<new_xml></new_xml>"
                "data": "[data_id]",
                "data_structure_element_root": "[data_structure_element_root_id]"
            }

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Updated  data structure
            - code: 400
              content: Validation error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)

            # Build serializer
            serializer = self.serializer(
                instance=data_structure_object,
                data=request.data,
                partial=True,
                context={"request": request},
            )

            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()

            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Authentication tests for Data Structure REST API
"""
from django.test import SimpleTestCase
from unittest.mock import patch
from rest_framework import status

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_curate_app.components.curate_data_structure.api as data_structure_api
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.rest.curate_data_structure import (
    views as data_structure_rest_views,
)
from core_curate_app.rest.curate_data_structure.admin_serializers import (
    CurateDataStructureAdminSerializer,
)
from core_curate_app.rest.curate_data_structure.serializers import (
    CurateDataStructureSerializer,
)


class TestDataStructureListAdminPostPermissions(SimpleTestCase):
    """
    Test Data Structure List Admin Post Permissions
    """

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:
        """

        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """
        test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(self):
        """
        test_staff_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CurateDataStructureAdminSerializer, "data")
    @patch.object(CurateDataStructureAdminSerializer, "save")
    @patch.object(CurateDataStructureAdminSerializer, "is_valid")
    def test_superuser_returns_http_201(
        self,
        data_structure_serializer_data,
        data_structure_serializer_save,
        data_structure_serializer_valid,
    ):
        """
        test_superuser_returns_http_201

        Returns:

        """
        data_structure_serializer_valid.return_value = True
        data_structure_serializer_save.return_value = None
        data_structure_serializer_data.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestDataStructureListAdminGetPermissions(SimpleTestCase):
    """
    Test Data Structure List Admin Get Permissions
    """

    def test_anonymous_returns_http_403(self):
        """
        test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """
        test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(self):
        """
        test_staff_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CurateDataStructure, "get_all")
    def test_superuser_returns_http_200(self, get_all):
        """
        test_superuser_returns_http_200

        Returns:

        """
        get_all.return_value = {}
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataStructureListGetPermissions(SimpleTestCase):
    """
    Test Data Structure List Get Permissions
    """

    def test_anonymous_returns_http_403(self):
        """
        test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CurateDataStructure, "get_all_by_user")
    def test_authenticated_returns_http_200(self, get_all):
        """
        test_authenticated_returns_http_200

        Returns:

        """
        get_all.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(CurateDataStructure, "get_all_by_user")
    def test_superuser_returns_http_200(self, get_all):
        """
        test_superuser_returns_http_200

        Returns:

        """
        get_all.return_value = {}
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataStructureDetailGetPermissions(SimpleTestCase):
    """
    Test Data Structure Detail Get Permissions
    """

    def test_anonymous_returns_http_403(self):
        """
        test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CurateDataStructureSerializer, "data")
    @patch.object(data_structure_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self, mock_data_structure_api_get_by_id, mock_data_structure_serializer
    ):
        """
        test_authenticated_returns_http_200

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = []
        mock_data_structure_serializer.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": "0"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(CurateDataStructureSerializer, "data")
    @patch.object(data_structure_api, "get_by_id")
    def test_staff_returns_http_200(
        self, mock_data_structure_api_get_by_id, mock_data_structure_serializer
    ):
        """
        test_staff_returns_http_200

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = []
        mock_data_structure_serializer.return_value = []

        mock_user = create_mock_user(
            "1",
            is_staff=True,
        )

        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": "0"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataStructureDetailDeletePermissions(SimpleTestCase):
    """
    Test Data Structure Detail Delete Permissions
    """

    def test_anonymous_returns_http_403(self):
        """
        test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_structure_api, "delete")
    @patch.object(data_structure_api, "get_by_id")
    def test_authenticated_returns_http_204(
        self, mock_data_structure_api_get_by_id, mock_data_structure_api_delete
    ):
        """
        test_authenticated_returns_http_204

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = None
        mock_data_structure_api_delete.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(data_structure_api, "delete")
    @patch.object(data_structure_api, "get_by_id")
    def test_staff_returns_http_204(
        self, mock_data_structure_api_get_by_id, mock_data_structure_api_delete
    ):
        """
        test_staff_returns_http_204

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = None
        mock_data_structure_api_delete.return_value = None

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestDataStructureDetailPatchPermissions(SimpleTestCase):
    """
    Test Data Structure Detail Patch Permissions
    """

    def test_anonymous_returns_http_403(self):
        """
        test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CurateDataStructureSerializer, "data")
    @patch.object(CurateDataStructureSerializer, "save")
    @patch.object(CurateDataStructureSerializer, "is_valid")
    @patch.object(data_structure_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_data_structure_api_get_by_id,
        mock_data_structure_serializer,
        mock_data_structure_serializer_save,
        mock_data_structure_serializer_is_valid,
    ):
        """
        test_authenticated_returns_http_200

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = []
        mock_data_structure_serializer_is_valid.return_value = True
        mock_data_structure_serializer_save.return_value = None
        mock_data_structure_serializer.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(CurateDataStructureSerializer, "data")
    @patch.object(CurateDataStructureSerializer, "save")
    @patch.object(CurateDataStructureSerializer, "is_valid")
    @patch.object(data_structure_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_structure_api_get_by_id,
        mock_data_structure_serializer_is_valid,
        mock_data_structure_serializer_save,
        mock_data_structure_serializer,
    ):
        """
        test_staff_returns_http_200

        Returns:

        """
        mock_data_structure_api_get_by_id.return_value = []
        mock_data_structure_serializer_is_valid.return_value = True
        mock_data_structure_serializer_save.return_value = None
        mock_data_structure_serializer.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Migration
"""

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    """Migration"""

    initial = True

    dependencies = [
        ("core_parser_app", "0001_initial"),
        ("core_main_app", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Curate",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_curate_app",
                "permissions": (
                    ("access_curate", "Can access curate"),
                    ("view_data_save_repo", "Can view data save repo"),
                    (
                        "access_curate_data_structure",
                        "Can access curate data structure",
                    ),
                ),
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CurateDataStructure",
            fields=[
                (
                    "datastructure_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core_parser_app.datastructure",
                    ),
                ),
                ("form_string", models.TextField(blank=True)),
                (
                    "data",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_app.data",
                    ),
                ),
            ],
            bases=("core_parser_app.datastructure",),
        ),
    ]



""" Initialize permissions for core curate app.
"""
import logging

from django.contrib.auth.models import Group, Permission
import core_main_app.permissions.rights as main_rights
import core_curate_app.permissions.rights as curate_rights


logger = logging.getLogger(__name__)


def init_permissions():
    """Initialization of groups and permissions."""
    try:
        # Get or Create the default group
        default_group, created = Group.objects.get_or_create(
            name=main_rights.DEFAULT_GROUP
        )

        # Get curate permissions
        curate_access_perm = Permission.objects.get(
            codename=curate_rights.CURATE_ACCESS
        )
        curate_view_data_save_repo_perm = Permission.objects.get(
            codename=curate_rights.CURATE_VIEW_DATA_SAVE_REPO
        )

        curate_data_structure_access_perm = Permission.objects.get(
            codename=curate_rights.CURATE_DATA_STRUCTURE_ACCESS
        )

        # Add permissions to default group
        default_group.permissions.add(
            curate_access_perm,
            curate_view_data_save_repo_perm,
            curate_data_structure_access_perm,
        )

    except Exception as exception:
        logger.error(
            "Impossible to init curate permissions: %s", str(exception)
        )



""" Test access to views
"""

from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.storage.fallback import FallbackStorage
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import RequestFactory
from tests.components.curate_data_structure.fixtures.fixtures import (
    DataStructureFixtures,
)

from core_curate_app.views.common.views import DraftContentEditor
from core_curate_app.views.user import ajax as curate_ajax
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user

fixture_data_structure = DataStructureFixtures()


class TestDraftContentEditorView(MongoIntegrationBaseTestCase):
    """Test Draft Content Editor View"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = DataStructureFixtures()
        self.fixture.insert_data()

    def test_get_assets(self):
        """test_get_assets

        Returns:

        """
        instance = DraftContentEditor()
        DraftContentEditor._get_assets(instance)

    def test_anonymous_user_can_not_access_to_draft(self):
        """test_anonymous_user_can_not_access_to_draft

        Returns:

        """
        request = self.factory.get("core_curate_app_xml_text_editor_view")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.data_structure_1.id)}
        response = DraftContentEditor.as_view()(request)
        self.assertTrue(
            self.fixture.data_structure_1.name not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_user_can_not_access_to_draft_if_not_found(self):
        """test_user_can_not_access_to_draft_if_not_found

        Returns:

        """
        request = self.factory.get("core_curate_app_xml_text_editor_view")
        request.GET = {"id": "-1"}
        request.user = self.user1
        response = DraftContentEditor.as_view()(request)
        self.assertTrue("Error 404" in response.content.decode())

    def test_user_can_not_access_to_draft_if_missing_param(self):
        """test_user_can_not_access_to_draft_if_missing_param

        Returns:

        """
        request = self.factory.get("core_curate_app_xml_text_editor_view")
        request.user = self.user1
        response = DraftContentEditor.as_view()(
            request,
        )
        self.assertEqual(response.status_code, 200)

    def test_user_can_access_to_draft_if_owner(self):
        """test_user_can_access_a_draft_if_owner

        Returns:

        """
        request = self.factory.get("core_curate_app_xml_text_editor_view")
        request.GET = {"id": str(self.fixture.data_structure_1.id)}
        request.user = self.user1
        response = DraftContentEditor.as_view()(
            request,
        )
        self.assertEqual(response.status_code, 200)

    def test_user_can_save_xml_content(self):
        """test_user_can_save_xml_content

        Returns:

        """
        data = {
            "content": "<tag></tag>",
            "action": "save",
            "document_id": str(self.fixture.data_structure_1.id),
            "id": str(self.fixture.data_structure_1.id),
        }
        request = self.factory.post(
            "core_curate_app_xml_text_editor_view", data
        )
        setattr(request, "session", "session")
        messages = FallbackStorage(request)
        setattr(request, "_messages", messages)
        request.user = self.user1
        response = DraftContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_save_xml_content_returns_acl_error(self):
        """test_user_save_xml_content_returns_acl_error

        Returns:

        """
        data = {
            "content": "<tag></tag>",
            "action": "save",
            "document_id": str(self.fixture.data_structure_1.id),
            "id": str(self.fixture.data_structure_1.id),
        }
        request = self.factory.post(
            "core_curate_app_xml_text_editor_view", data
        )
        request.user = self.anonymous
        response = DraftContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 403)

    def test_user_save_xml_content_returns_dne_error(self):
        """test_user_save_xml_content_returns_dne_error

        Returns:

        """
        data = {
            "content": "<tag></tag>",
            "action": "save",
            "document_id": "-1",
            "id": "-1",
        }
        request = self.factory.post(
            "core_curate_app_xml_text_editor_view", data
        )
        request.user = self.user1
        response = DraftContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_save_xml_content_returns_error(self):
        """test_user_save_xml_content_returns_error

        Returns:

        """
        data = {
            "action": "save",
            "document_id": "-1",
            "id": "-1",
        }
        request = self.factory.post(
            "core_curate_app_xml_text_editor_view", data
        )
        request.user = self.user1
        response = DraftContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)


class TestStartCuratePost(MongoIntegrationBaseTestCase):
    """Test Start Curate Post"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.fixture = DataStructureFixtures()
        self.fixture.insert_data()

    def test_start_curate_new_returns_http_200(self):
        """test_start_curate_new_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test",
            "forms": "",
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)

    def test_start_curate_new_in_text_editor_returns_http_200(self):
        """test_start_curate_new_in_text_editor_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test",
            "forms": "",
            "text_editor": "on",
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)

    def test_start_curate_upload_returns_http_200(self):
        """test_start_curate_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "forms": "",
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        content = "<tag></tag>"
        files = SimpleUploadedFile("test.xml", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)

    def test_start_curate_in_text_editor_upload_returns_http_200(self):
        """test_start_curate_in_text_editor_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "forms": "",
            "text_editor": "on",
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        content = "<tag></tag>"
        files = SimpleUploadedFile("test.xml", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)

    def test_start_curate_open_form_returns_http_200(self):
        """test_start_curate_open_form_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "open",
            "document_name": "test",
            "forms": str(self.fixture.data_structure_1.id),
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)

    def test_start_curate_open_form_in_text_editor_returns_http_200(self):
        """test_start_curate_open_form_in_text_editor_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "open",
            "document_name": "test",
            "forms": str(self.fixture.data_structure_1.id),
            "text_editor": "on",
            "hidden_value": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)



""" ACL Test Data Structure
"""
from django.contrib.auth.models import AnonymousUser
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_parser_app.components.data_structure.models import (
    DataStructureElement,
)
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
import core_curate_app.components.curate_data_structure.api as curate_data_structure_api

from tests.components.curate_data_structure.fixtures.fixtures import (
    DataStructureFixtures,
)


fixture_data_structure = DataStructureFixtures()


class TestCurateDataStructureGetById(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Get ById
    """

    fixture = fixture_data_structure

    def test_get_by_id_as_superuser_returns_data_structure(self):
        """
        test_get_by_id_as_superuser_returns_data_structure

        Returns:

        """
        data_structure_id = self.fixture.data_structure_1.id
        mock_user = create_mock_user(
            self.fixture.data_structure_1.user,
            is_staff=True,
            is_superuser=True,
        )
        data_structure = curate_data_structure_api.get_by_id(
            data_structure_id, mock_user
        )
        self.assertTrue(isinstance(data_structure, CurateDataStructure))

    def test_get_by_id_as_owner_returns_data_structure(self):
        """
        test_get_by_id_as_owner_returns_data_structure

        Returns:

        """
        data_structure_id = self.fixture.data_structure_1.id
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        data_structure = curate_data_structure_api.get_by_id(
            data_structure_id, mock_user
        )
        self.assertTrue(isinstance(data_structure, CurateDataStructure))

    def test_get_by_id_as_user_non_owner_raises_error(self):
        """
        test_get_by_id_as_user_non_owner_raises_error

        Returns:

        """
        data_structure_id = self.fixture.data_structure_1.id
        mock_user = create_mock_user(self.fixture.data_structure_3.user)
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_by_id(data_structure_id, mock_user)

    def test_get_by_id_as_anonymous_user_raises_error(self):
        """
        test_get_by_id_as_anonymous_user_raises_error

        Returns:

        """
        data_structure_id = self.fixture.data_structure_1.id
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_by_id(
                data_structure_id, AnonymousUser()
            )


class TestCurateDataStructureGetAll(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Get All
    """

    fixture = fixture_data_structure

    def test_get_all_as_superuser_returns_all_data_structure(self):
        """
        test_get_all_as_superuser_returns_all_data_structure

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)
        result = curate_data_structure_api.get_all(mock_user)
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )

    def test_get_all_as_user_raises_error(self):
        """
        test_get_all_as_user_raises_error

        Returns:

        """
        mock_user = create_mock_user("1")
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all(mock_user)

    def test_get_all_as_anonymous_user_raises_error(self):
        """
        test_get_all_as_anonymous_user_raises_error

        Returns:

        """
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all(AnonymousUser())


class TestCurateDataStructureGetAllExceptUserIdWithNoData(
    MongoIntegrationBaseTestCase
):
    """
    Test Curate Data Structure Get All Except User Id With No Data
    """

    fixture = fixture_data_structure

    def test_get_all_except_user_id_with_no_data_as_superuser_returns_all_data_structure_except_user(
        self,
    ):
        """
        test_get_all_except_user_id_with_no_data_as_superuser_returns_all_data_structure_except_user

        Returns:

        """
        mock_user = create_mock_user("4", is_staff=True, is_superuser=True)
        user_id = self.fixture.data_structure_3.user
        result = curate_data_structure_api.get_all_except_user_id_with_no_data(
            user_id, mock_user
        )
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )
        self.assertTrue(
            all(
                (item.user != self.fixture.data_structure_3.user)
                for item in result
            )
        )
        self.assertTrue(all((item.data is None) for item in result))

    def test_get_all_except_user_id_with_no_data_as_user_raises_error(self):
        """
        test_get_all_except_user_id_with_no_data_as_user_raises_error

        Returns:

        """
        mock_user = create_mock_user("1")
        user_id = self.fixture.data_structure_3.user
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_except_user_id_with_no_data(
                user_id, mock_user
            )

    def test_get_all_except_user_id_with_no_data_as_anonymous_user_raises_error(
        self,
    ):
        """
        test_get_all_except_user_id_with_no_data_as_anonymous_user_raises_error

        Returns:

        """
        user_id = self.fixture.data_structure_3.user
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_except_user_id_with_no_data(
                user_id, AnonymousUser()
            )


class TestCurateDataStructureDelete(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Delete
    """

    fixture = fixture_data_structure

    def test_delete_others_data_structure_as_superuser_deletes_data_structure(
        self,
    ):
        """
        test_delete_others_data_structure_as_superuser_deletes_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_3
        mock_user = create_mock_user(
            self.fixture.data_structure_1.user,
            is_staff=True,
            is_superuser=True,
        )
        curate_data_structure_api.delete(data_structure, mock_user)

    def test_delete_own_data_structure_deletes_data_structure(self):
        """
        test_delete_own_data_structure_deletes_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        curate_data_structure_api.delete(data_structure, mock_user)

    def test_delete_others_data_structure_as_user_raises_error(self):
        """
        test_delete_others_data_structure_as_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_3
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.delete(data_structure, mock_user)

    def test_delete_others_data_structure_as_anonymous_user_raises_error(self):
        """
        test_delete_others_data_structure_as_anonymous_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_3
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.delete(data_structure, AnonymousUser())


class TestCurateDataStructureUpdateDataStructureRoot(
    MongoIntegrationBaseTestCase
):
    """
    Test Curate Data Structure Update Data Structure Root
    """

    fixture = fixture_data_structure

    def test_update_others_data_structure_root_as_superuser_updates_data_structure(
        self,
    ):
        """
        test_update_others_data_structure_root_as_superuser_updates_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        new_data_structure_element_root = _get_data_structure_element(
            self.fixture.data_structure_3.user, self.fixture.data_structure_3
        )
        mock_user = create_mock_user(
            self.fixture.data_structure_2.user,
            is_staff=True,
            is_superuser=True,
        )
        result = curate_data_structure_api.update_data_structure_root(
            data_structure, new_data_structure_element_root, mock_user
        )
        self.assertTrue(isinstance(result, CurateDataStructure))
        self.assertTrue(
            result.data_structure_element_root, new_data_structure_element_root
        )

    def test_update_own_data_structure_root_updates_data_structure(self):
        """
        test_update_own_data_structure_root_updates_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        new_data_structure_element_root = _get_data_structure_element(
            self.fixture.data_structure_2.user, self.fixture.data_structure_2
        )
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        result = curate_data_structure_api.update_data_structure_root(
            data_structure, new_data_structure_element_root, mock_user
        )
        self.assertTrue(isinstance(result, CurateDataStructure))
        self.assertTrue(
            result.data_structure_element_root, new_data_structure_element_root
        )

    def test_update_others_data_structure_root_as_user_raises_error(self):
        """
        test_update_others_data_structure_root_as_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        new_data_structure_element_root = self.fixture.data_structure_2.id
        mock_user = create_mock_user(self.fixture.data_structure_3.user)
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.update_data_structure_root(
                data_structure, new_data_structure_element_root, mock_user
            )

    def test_update_others_data_structure_root_as_anonymous_user_raises_error(
        self,
    ):
        """
        test_update_others_data_structure_root_as_anonymous_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        new_data_structure_element_root = self.fixture.data_structure_2.id
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.update_data_structure_root(
                data_structure,
                new_data_structure_element_root,
                AnonymousUser(),
            )


class TestCurateDataStructureCreateOrUpdate(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Create Or Update
    """

    fixture = fixture_data_structure

    def test_upsert_others_data_structure_as_superuser_updates_data_structure(
        self,
    ):
        """
        test_upsert_others_data_structure_as_superuser_updates_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_3
        data_structure.name = "new_name_3"
        mock_user = create_mock_user(
            self.fixture.data_structure_1.user,
            is_staff=True,
            is_superuser=True,
        )
        result = curate_data_structure_api.upsert(data_structure, mock_user)
        self.assertTrue(isinstance(result, CurateDataStructure))
        self.assertTrue(data_structure.name, result.name)

    def test_upsert_own_data_structure_updates_data_structure(self):
        """
        test_upsert_own_data_structure_updates_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        data_structure.name = "new_name_1"
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        result = curate_data_structure_api.upsert(data_structure, mock_user)
        self.assertTrue(isinstance(result, CurateDataStructure))
        self.assertTrue(data_structure.name, result.name)

    def test_upsert_others_data_structure_as_user_raises_error(self):
        """
        test_upsert_others_data_structure_as_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        data_structure.name = "new_name_1"
        mock_user = create_mock_user(self.fixture.data_structure_3.user)
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.upsert(data_structure, mock_user)

    def test_upsert_data_structure_as_anonymous_user_raises_error(self):
        """
        test_upsert_data_structure_as_anonymous_user_raises_error

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        data_structure.name = "new_name_1"
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.upsert(data_structure, AnonymousUser())


class TestCurateDataStructureGetAllWithNoData(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Get All With No Data
    """

    fixture = fixture_data_structure

    def test_get_all_with_no_data_as_superuser_returns_all_data_structure(
        self,
    ):
        """
        test_get_all_with_no_data_as_superuser_returns_all_data_structure

        Returns:

        """
        mock_user = create_mock_user("4", is_staff=True, is_superuser=True)
        result = curate_data_structure_api.get_all_with_no_data(mock_user)
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )
        self.assertTrue(len(result), 3)

    def test_get_all_with_no_data_as_user_raises_error(self):
        """
        test_get_all_with_no_data_as_user_raises_error

        Returns:

        """
        mock_user = create_mock_user("1")
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_with_no_data(mock_user)

    def test_get_all_with_no_data_as_anonymous_user_raises_error(self):
        """
        test_get_all_with_no_data_as_anonymous_user_raises_error

        Returns:

        """
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_with_no_data(AnonymousUser())


class TestCurateDataStructureGetByDataId(MongoIntegrationBaseTestCase):
    """
    Test Curate Data Structure Get ById
    """

    fixture = fixture_data_structure

    def test_get_by_data_id_as_superuser_returns_data_structure(self):
        """
        test_get_by_data_id_as_superuser_returns_data_structure

        Returns:

        """

        mock_user = create_mock_user(
            self.fixture.data_structure_3.user,
            is_staff=True,
            is_superuser=True,
        )
        data_structure = curate_data_structure_api.get_by_data_id(
            self.fixture.data.id, mock_user
        )
        self.assertTrue(isinstance(data_structure, CurateDataStructure))
        self.assertEquals(self.fixture.data.id, data_structure.data.id)

    def test_get_by_data_id_as_owner_returns_data_structure(self):
        """
        test_get_by_data_id_as_owner_returns_data_structure

        Returns:

        """
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        data_structure = curate_data_structure_api.get_by_data_id(
            self.fixture.data.id, mock_user
        )
        self.assertTrue(isinstance(data_structure, CurateDataStructure))
        self.assertEquals(self.fixture.data.id, data_structure.data.id)

    def test_get_by_data_id_as_user_non_owner_raises_error(self):
        """
        test_get_by_data_id_as_user_non_owner_raises_error

        Returns:

        """
        mock_user = create_mock_user(self.fixture.data_structure_3.user)
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_by_data_id(
                self.fixture.data.id, mock_user
            )

    def test_get_by_data_id_as_anonymous_user_raises_error(self):
        """
        test_get_by_data_id_as_anonymous_user_raises_error

        Returns:

        """
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_by_data_id(
                self.fixture.data.id, AnonymousUser()
            )


class TestDataStructureChangeOwner(MongoIntegrationBaseTestCase):
    """
    Test Data Structure Change Owner
    """

    fixture = fixture_data_structure

    def test_change_owner_from_owner_to_owner_ok(self):
        """
        test_change_owner_from_owner_to_owner_ok

        Returns:

        """
        mock_owner = create_mock_user(self.fixture.data_structure_1.user)
        curate_data_structure_api.change_owner(
            document=fixture_data_structure.data_structure_1,
            new_user=mock_owner,
            user=mock_owner,
        )

    def test_change_owner_from_owner_to_user_ok(self):
        """
        test_change_owner_from_owner_to_user_ok

        Returns:

        """
        mock_owner = create_mock_user(self.fixture.data_structure_1.user)
        mock_user = create_mock_user("2")
        curate_data_structure_api.change_owner(
            document=fixture_data_structure.data_structure_1,
            new_user=mock_user,
            user=mock_owner,
        )

    def test_change_owner_from_user_to_user_raises_exception(self):
        """
        test_change_owner_from_user_to_user_raises_exception

        Returns:

        """
        mock_owner = create_mock_user("0")
        mock_user = create_mock_user("2")
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.change_owner(
                document=fixture_data_structure.data_structure_1,
                new_user=mock_user,
                user=mock_owner,
            )

    def test_change_owner_from_anonymous_to_user_raises_exception(self):
        """
        test_change_owner_from_anonymous_to_user_raises_exception

        Returns:

        """

        mock_user = create_mock_user("2")
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.change_owner(
                document=fixture_data_structure.data_structure_1,
                new_user=mock_user,
                user=AnonymousUser(),
            )

    def test_change_owner_as_superuser_ok(self):
        """
        test_change_owner_as_superuser_ok

        Returns:

        """
        mock_user = create_mock_user("2", is_superuser=True)
        curate_data_structure_api.change_owner(
            document=fixture_data_structure.data_structure_1,
            new_user=mock_user,
            user=mock_user,
        )


def _get_data_structure_element(user, data_structure):
    """Return a data structure element

    Args:
        user:
        data_structure:

    Returns:

    """
    data_structure_element = DataStructureElement(
        user=user,
        tag="tag",
        value="value",
        options={},
        data_structure=data_structure,
    )
    data_structure_element.save()
    return data_structure_element



"""Curate Data Structure models
"""
from django.core.exceptions import ObjectDoesNotExist
from django.db import models, IntegrityError
from django.db.models.signals import pre_delete
from core_main_app.commons import exceptions
from core_main_app.utils.validation.regex_validation import (
    not_empty_or_whitespaces,
)
from core_main_app.components.data.models import Data
from core_parser_app.components.data_structure.models import DataStructure
from core_curate_app.permissions import rights


class CurateDataStructure(DataStructure):
    """Curate data structure."""

    form_string = models.TextField(blank=True)
    data = models.ForeignKey(
        Data, blank=True, on_delete=models.CASCADE, null=True
    )

    @staticmethod
    def get_permission():
        """get_permission

        Returns:

        """
        return f"{rights.CURATE_CONTENT_TYPE}.{rights.CURATE_DATA_STRUCTURE_ACCESS}"

    def save_object(self):
        """Custom save

        Returns:

        """
        try:
            self.clean()
            return self.save()
        except IntegrityError:
            raise exceptions.NotUniqueError(
                "Unable to save the document: not unique."
            )
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_id(data_structure_id):
        """Return the object with the given id.

        Args:
            data_structure_id:

        Returns:
            Curate Data Structure (obj): CurateDataStructure object with the given id

        """
        try:
            return CurateDataStructure.objects.get(pk=str(data_structure_id))
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_none():
        """Return None object, used by forms.

        Returns:

        """
        return CurateDataStructure.objects.none()

    @staticmethod
    def get_all():
        """Return all curate data structure api.

        Returns:

        """
        return CurateDataStructure.objects.all()

    @staticmethod
    def get_all_by_user_id_and_template_id(user_id, template_id):
        """Return all template version managers with user set to None.

        Returns:

        """
        return CurateDataStructure.objects.filter(
            user=str(user_id), template=str(template_id)
        ).all()

    @staticmethod
    def get_by_user_id_and_template_id_and_name(user_id, template_id, name):
        """Return the curate data structure with user, template id and name.

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(
                user=str(user_id), template=str(template_id), name=name
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_all_by_user_id_with_no_data(user_id):
        """
        Return all the curate date structure of the user, with no data.

        Args: user_id:
        Return:
        """
        return CurateDataStructure.objects.filter(
            user=str(user_id), data__isnull=True
        ).all()

    @staticmethod
    def get_all_except_user_id_with_no_data(user_id):
        """Return all the curate date structure except the one of the user, with no data.

        Args: user_id:
        Return:
        """
        return (
            CurateDataStructure.objects.exclude(user=str(user_id))
            .filter(data__isnull=True)
            .all()
        )

    @staticmethod
    def get_all_by_user_id_and_template_id_with_no_data(user_id, template_id):
        """

        Args:
            user_id:
            template_id:

        Returns:

        """
        return CurateDataStructure.objects.filter(
            user=str(user_id), template=str(template_id), data__isnull=True
        ).all()

    @staticmethod
    def get_all_with_no_data():
        """Returns all curate data structure api with no link to a data.

        Args:

        Returns:

        """
        return CurateDataStructure.objects.filter(data__isnull=True).all()

    @staticmethod
    def get_all_by_user(user_id):
        """Return all curate data structure by user.

        Returns:

        """
        return CurateDataStructure.objects.filter(user=str(user_id)).all()

    @staticmethod
    def get_by_data_id(data_id):
        """Return the curate data structure with the given data id

        Args:
            data_id:

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(data=str(data_id))
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_data_structure_element_root(data_structure_element_root):
        """Return the curate data structure with the given data id

        Args:
            data_structure_element_root:

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(
                data_structure_element_root=str(data_structure_element_root.id)
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    def clean(self):
        """Clean before saving

        Returns:

        """
        not_empty_or_whitespaces(self.name)
        self.name = self.name.strip()


# Connect signals
pre_delete.connect(DataStructure.pre_delete, sender=CurateDataStructure)



""" User curate forms
"""
from django import forms

from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE
from core_main_app.utils.labels import get_form_label
import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
from django.core.exceptions import ValidationError


class NewForm(forms.Form):
    """Form to start curating from an empty form."""

    document_name = forms.CharField(
        label="",
        max_length=100,
        required=True,
        widget=forms.TextInput(attrs={"class": "form-control"}),
    )


class FormDataModelChoiceField(forms.ModelChoiceField):
    """Choice Field to select an existing form."""

    def label_from_instance(self, obj):
        """Return label.

        Args:
            obj:

        Returns:

        """
        return obj.name


class OpenForm(forms.Form):
    """Form to open an existing form."""

    forms = FormDataModelChoiceField(
        label="",
        queryset=curate_data_structure_api.get_none(),
        widget=forms.Select(attrs={"class": "form-control"}),
    )

    def __init__(self, *args, **kwargs):
        if "forms" in kwargs:
            queryset = kwargs.pop("forms")
        else:
            queryset = curate_data_structure_api.get_none()
        super().__init__(*args, **kwargs)
        self.fields["forms"].queryset = queryset


def _file_size_validator(value):
    """Check size of uploaded file

    Args:
        value:

    Returns:

    """
    if value.size > MAX_DOCUMENT_EDITING_SIZE:
        raise ValidationError(
            "The file is too large (MAX_DOCUMENT_EDITING_SIZE)."
        )


class UploadForm(forms.Form):
    """Form to start curating from a file."""

    file = forms.FileField(
        label="",
        widget=forms.FileInput(
            attrs={"class": "form-control", "accept": ".xml"}
        ),
        validators=[_file_size_validator],
    )
    direct_upload = forms.BooleanField(
        label="",
        widget=forms.CheckboxInput(attrs={"hidden": "true"}),
        required=False,
    )
    text_editor = forms.BooleanField(
        label="",
        widget=forms.CheckboxInput(attrs={"hidden": "true"}),
        required=False,
    )


class CancelChangesForm(forms.Form):
    """Cancel changes form."""

    CANCEL_CHOICES = [
        ("revert", "Revert to my previously Saved " + get_form_label()),
        ("return", "Return to Add Resources"),
    ]

    cancel = forms.ChoiceField(
        label="", choices=CANCEL_CHOICES, widget=forms.RadioSelect()
    )


class HiddenFieldsForm(forms.Form):
    """Form for hidden fields."""

    hidden_value = forms.CharField(widget=forms.HiddenInput(), required=True)

    def __init__(self, *args, **kwargs):
        value = ""
        if "hidden_value" in kwargs:
            value = kwargs.pop("hidden_value")
        super().__init__(*args, **kwargs)
        self.fields["hidden_value"].initial = value



"""Curate app user views
"""
import logging
from builtins import any
from typing import Dict, List

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy, reverse
from django.utils.decorators import method_decorator
from django.views import View

import core_main_app.components.template_version_manager.api as template_version_manager_api
import core_main_app.components.template.api as template_api
from core_main_app.components.lock import api as lock_api
from core_main_app.utils import decorators
from core_main_app.commons.exceptions import (
    LockError,
    ModelError,
    DoesNotExist,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.access_control.api import check_can_write

from core_main_app.utils.file import get_file_http_response
from core_main_app.utils.labels import get_form_label
from core_main_app.utils.rendering import render
from core_main_app.utils.boolean import to_bool
from core_main_app.utils.xml import format_content_xml
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_parser_app.tools.parser.exceptions import ParserError
from core_parser_app.tools.parser.renderer.list import ListRenderer
from core_parser_app.tools.parser.renderer.xml import XmlRenderer

from core_curate_app.components.curate_data_structure import (
    api as curate_data_structure_api,
)
from core_curate_app.permissions import rights
from core_curate_app.settings import (
    INSTALLED_APPS,
    ENABLE_XML_ENTITIES_TOOLTIPS,
)
from core_curate_app.utils.parser import get_parser

logger = logging.getLogger(__name__)

# TODO: Add a view for the registry. Ajax code need to be refactored


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    login_url=reverse_lazy("core_main_app_login"),
)
def index(request):
    """Page that allows to select a template to start curating.

    Args:
        request:

    Returns:

    """
    assets = {
        "js": [
            {
                "path": "core_curate_app/user/js/select_template.js",
                "is_raw": False,
            },
            {
                "path": "core_curate_app/user/js/select_template.raw.js",
                "is_raw": True,
            },
        ],
        "css": [
            "core_curate_app/user/css/common.css",
            "core_curate_app/user/css/style.css",
        ],
    }

    global_active_template_list = (
        template_version_manager_api.get_active_global_version_manager(
            request=request
        )
    )
    user_active_template_list = (
        template_version_manager_api.get_active_version_manager_by_user_id(
            request=request
        )
    )

    context = {
        "templates_version_manager": global_active_template_list,
        "userTemplates": user_active_template_list,
    }

    # Set page title
    context.update({"page_title": "Curate"})

    return render(
        request,
        "core_curate_app/user/curate.html",
        # modals=modals,
        assets=assets,
        context=context,
    )


# FIXME: allow reopening a form with unsaved changes (may be temporary until
#  curate workflow redesign)
class EnterDataView(View):
    """Page that allows to enter a data view."""

    def __init__(self):
        super().__init__()
        self.assets = {
            "js": [
                {
                    "path": "core_main_app/common/js/debounce.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/elementViewport.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/enter_data.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/enter_data.raw.js",
                    "is_raw": True,
                },
                {"path": "core_parser_app/js/modules.js", "is_raw": False},
                {"path": "core_parser_app/js/modules.raw.js", "is_raw": True},
                {
                    "path": "core_main_app/common/js/XMLTree.js",
                    "is_raw": False,
                },
                {"path": "core_parser_app/js/autosave.js", "is_raw": False},
                {"path": "core_parser_app/js/autosave.raw.js", "is_raw": True},
                {"path": "core_parser_app/js/buttons.js", "is_raw": False},
                {
                    "path": "core_curate_app/user/js/buttons.raw.js",
                    "is_raw": True,
                },
                {"path": "core_parser_app/js/choice.js", "is_raw": False},
                {
                    "path": "core_curate_app/user/js/choice.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_curate_app/user/js/download.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/modals/download.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/data_detail.js",
                    "is_raw": False,
                },
            ],
            "css": [
                "core_curate_app/user/css/common.css",
                "core_curate_app/user/css/xsd_form.css",
                "core_parser_app/css/use.css",
            ],
        }

        if ENABLE_XML_ENTITIES_TOOLTIPS:
            self.assets["js"].append(
                {
                    "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                    "is_raw": False,
                }
            )

        self.modals = [
            "core_curate_app/user/data-entry/modals/cancel-changes.html",
            "core_curate_app/user/data-entry/modals/cancel-form.html",
            "core_curate_app/user/data-entry/modals/clear-fields.html",
            "core_main_app/common/modals/download-options.html",
            "core_curate_app/user/data-entry/modals/save-form.html",
            "core_curate_app/user/data-entry/modals/use-validation.html",
            "core_curate_app/user/data-entry/modals/xml-error.html",
            "core_curate_app/user/data-entry/modals/xml-valid.html",
        ]

    def build_context(
        self, request, curate_data_structure, reload_unsaved_changes
    ):
        """Build the context of the view

        Args:
            request:
            curate_data_structure:
            reload_unsaved_changes:

        Returns:

        """
        if reload_unsaved_changes:
            # get root element from the data structure
            root_element = curate_data_structure.data_structure_element_root
        else:
            # if form string provided, use it to generate the form
            xml_string = curate_data_structure.form_string

            root_element = generate_root_element(
                request, curate_data_structure, xml_string
            )

        # renders the form
        xsd_form = render_form(request, root_element)

        return {
            "edit": True if curate_data_structure.data is not None else False,
            "xsd_form": xsd_form,
            "data_structure": curate_data_structure,
        }

    @method_decorator(
        decorators.permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(
        self, request, curate_data_structure_id, reload_unsaved_changes=False
    ):
        """Load view to enter data.

        Args:
            request:
            curate_data_structure_id:
            reload_unsaved_changes:

        Returns:

        """
        curate_data_structure = None
        try:
            # Retrieve CurateDataStructure and lock the object for the current
            # user.
            curate_data_structure = _get_curate_data_structure_by_id(
                curate_data_structure_id, request
            )

            # Lock from database
            if curate_data_structure.data is not None:
                lock_api.set_lock_object(
                    curate_data_structure.data, request.user
                )

            # Build context
            context = self.build_context(
                request, curate_data_structure, reload_unsaved_changes
            )
            # Set page title
            context.update({"page_title": "Enter Data"})

            return render(
                request,
                "core_curate_app/user/data-entry/enter_data.html",
                assets=self.assets,
                context=context,
                modals=self.modals,
            )
        except (LockError, AccessControlError, ModelError, DoesNotExist) as ex:
            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(ex), "page_title": "Error"},
            )
        except ParserError as parser_exception:
            messages.add_message(
                self.request,
                messages.ERROR,
                str(parser_exception),
            )
            url = (
                reverse("core_curate_app_xml_text_editor_view")
                + f"?id={str(curate_data_structure_id)}"
            )
            return HttpResponseRedirect(url)
        except Exception as exception:
            try:
                # Unlock from database
                if (
                    curate_data_structure is not None
                    and curate_data_structure.data is not None
                ):
                    lock_api.remove_lock_on_object(
                        curate_data_structure.data, request.user
                    )
            except Exception as lock_exc:
                # CurateDataStructure not found, continue search
                logger.warning(
                    "'EnterDataView.get' threw an exception:%s", str(lock_exc)
                )

            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(exception), "page_title": "Error"},
            )


class ViewDataView(View):
    """
    Page that allows to view a data.

    Args:
        View:

    Returns:

    """

    def __init__(self):
        super().__init__()
        self.assets: Dict[str, List[any]] = {
            "js": [
                {
                    "path": "core_curate_app/user/js/view_data.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/view_data.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/XMLTree.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/download.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/modals/download.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/data_detail.js",
                    "is_raw": False,
                },
            ],
            "css": ["core_main_app/common/css/XMLTree.css"],
        }

        self.modals = [
            "core_curate_app/user/data-review/modals/save-error.html",
            "core_main_app/common/modals/download-options.html",
        ]

    def build_context(self, request, curate_data_structure):
        """Build XML string from CurateDataStructure

        Args:
            request:
            curate_data_structure:

        Returns:
        """
        xml_string = render_xml(
            request, curate_data_structure.data_structure_element_root
        )

        return {
            "edit": True if curate_data_structure.data is not None else False,
            "xml_string": xml_string,
            "data_structure": curate_data_structure,
        }

    @method_decorator(
        decorators.permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, curate_data_structure_id):
        """Get data view

        Args:
            request:
            curate_data_structure_id:

        Returns:
        """

        try:
            curate_data_structure = _get_curate_data_structure_by_id(
                curate_data_structure_id, request
            )

            if "core_file_preview_app" in INSTALLED_APPS:
                self.assets["js"].extend(
                    [
                        {
                            "path": "core_file_preview_app/user/js/file_preview.js",
                            "is_raw": False,
                        }
                    ]
                )
                self.assets["css"].append(
                    "core_file_preview_app/user/css/file_preview.css"
                )
                self.modals.append(
                    "core_file_preview_app/user/file_preview_modal.html"
                )

            # Build Context
            context = self.build_context(request, curate_data_structure)
            # Set page title
            context.update({"page_title": "View Data"})

            return render(
                request,
                "core_curate_app/user/data-review/view_data.html",
                assets=self.assets,
                context=context,
                modals=self.modals,
            )
        except Exception as exception:
            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(exception), "page_title": "Error"},
            )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    login_url=reverse_lazy("core_main_app_login"),
)
def download_current_xml(request, curate_data_structure_id):
    """Make the current XML document available for download.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    # get curate data structure
    curate_data_structure = _get_curate_data_structure_by_id(
        curate_data_structure_id, request
    )

    # generate xml string
    xml_data = render_xml(
        request, curate_data_structure.data_structure_element_root
    )

    # get pretty print bool
    prettify = request.GET.get("pretty_print", False)

    # prettify content
    if to_bool(prettify):
        xml_data = format_content_xml(xml_data)

    # build response with file
    return get_file_http_response(
        file_content=xml_data,
        file_name=curate_data_structure.name,
        content_type="application/xml",
        extension="xml",
    )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    login_url=reverse_lazy("core_main_app_login"),
)
def download_xsd(request, curate_data_structure_id):
    """Make the current XSD available for download.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    # get curate data structure
    curate_data_structure = _get_curate_data_structure_by_id(
        curate_data_structure_id, request
    )

    # get the template
    template = template_api.get_by_id(
        str(curate_data_structure.template.id), request=request
    )

    # get the template content
    content = template.content

    # get pretty print bool
    prettify = request.GET.get("pretty_print", False)

    # prettify content
    if to_bool(prettify):
        content = format_content_xml(content)

    # return the file
    return get_file_http_response(
        file_content=content,
        file_name=template.filename,
        content_type="application/xsd",
        extension=".xsd",
    )


def generate_form(
    xsd_string, xml_string=None, data_structure=None, request=None
):
    """Generate the form using the parser, returns the root element.

    Args:
        xsd_string:
        xml_string:
        data_structure:
        request:

    Returns:

    """
    # build parser
    parser = get_parser(request=request)
    # generate form
    root_element_id = parser.generate_form(
        xsd_string, xml_string, data_structure, request=request
    )
    # get the root element
    root_element = data_structure_element_api.get_by_id(
        root_element_id, request
    )

    return root_element


def render_form(request, root_element):
    """Render the form.

    Args:
        request:
        root_element:

    Returns:

    """
    # build a renderer
    renderer = ListRenderer(root_element, request)
    # render the form
    xsd_form = renderer.render()

    return xsd_form


def generate_root_element(request, curate_data_structure, xml_string):
    """Render the updated form.

    Args:
        request:
        curate_data_structure:
        xml_string:

    Returns:

    """
    # get xsd string from the template
    template = template_api.get_by_id(
        str(curate_data_structure.template.id), request=request
    )

    # get the root element
    root_element = generate_form(
        template.content,
        xml_string,
        data_structure=curate_data_structure,
        request=request,
    )

    # save the root element in the data structure
    curate_data_structure_api.update_data_structure_root(
        curate_data_structure, root_element, request.user
    )

    return root_element


def render_xml(request, root_element):
    """Render the XML.

    Args:
        request:
        root_element:

    Returns:

    """
    # build XML renderer
    xml_renderer = XmlRenderer(root_element, request)

    # generate xml data
    xml_data = xml_renderer.render()

    return xml_data


def _get_curate_data_structure_by_id(curate_data_structure_id, request):
    """Get the curate data structure by its id.

    Args: curate_data_structure_id:
          request:
    Returns:
    """

    # get data structure
    curate_data_structure = curate_data_structure_api.get_by_id(
        curate_data_structure_id, request.user
    )

    # If not link to a data, only ownership check
    if curate_data_structure.data is None:
        # check ownership
        _check_owner(request, accessed_object=curate_data_structure)
    # Check based on the data
    else:
        # check can write data
        _check_can_write_data(request, accessed_object=curate_data_structure)

    return curate_data_structure


def _check_owner(request, accessed_object):
    """Check if the object can be accessed by the user.

    Args:
        request:
        accessed_object:

    Returns:

    """
    # Super user can access everything
    if not request.user.is_superuser:
        # If not the owner of the accessed object
        if str(request.user.id) != accessed_object.user:
            raise AccessControlError(
                "You are not the owner of the "
                + get_form_label()
                + " that you are trying to access"
            )


def _check_can_write_data(request, accessed_object):
    """Check if the object can be accessed by the user in write mode.

    Args:
        request:
        accessed_object:

    Returns:

    """
    try:
        # Super user can access everything
        if not request.user.is_superuser:
            check_can_write(accessed_object.data, request.user)
    except AccessControlError as ace:
        raise ace



"""Parser util for curate app
"""
from core_parser_app.tools.parser.parser import XSDParser
from core_curate_app.settings import (
    PARSER_MIN_TREE,
    PARSER_IGNORE_MODULES,
    PARSER_COLLAPSE,
    PARSER_AUTO_KEY_KEYREF,
    PARSER_IMPLICIT_EXTENSION_BASE,
    PARSER_DOWNLOAD_DEPENDENCIES,
)


def get_parser(request=None):
    """Load configuration for the parser.

    Args:
        request:

    Returns:

    """
    return XSDParser(
        min_tree=PARSER_MIN_TREE,
        ignore_modules=PARSER_IGNORE_MODULES,
        collapse=PARSER_COLLAPSE,
        auto_key_keyref=PARSER_AUTO_KEY_KEYREF,
        implicit_extension_base=PARSER_IMPLICIT_EXTENSION_BASE,
        download_dependencies=PARSER_DOWNLOAD_DEPENDENCIES,
        request=request,
    )



"""
    Core curate exceptions
"""


class CurateAjaxError(Exception):
    """
    Exception raised by the curate package from views.
    """

    def __init__(self, message):
        self.message = message



"""AJAX views for the Curate app
"""
import json
import logging

from django.contrib import messages
from django.contrib.messages.storage.base import Message
from django.http.response import HttpResponseBadRequest, HttpResponse
from django.template import loader
from django.urls import reverse
from django.utils.html import escape
from lxml.etree import XMLSyntaxError

import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
import core_curate_app.views.user.forms as users_forms
from core_curate_app.common import exceptions as exceptions
from core_curate_app.common.exceptions import CurateAjaxError
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.permissions import rights as rights
from core_curate_app.utils.parser import get_parser
from core_curate_app.views.user.views import (
    generate_form,
    render_form,
    render_xml,
    generate_root_element,
)
from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.lock import api as lock_api
from core_main_app.components.template import api as template_api
from core_main_app.utils import decorators
from core_main_app.utils.labels import get_data_label, get_form_label
from core_main_app.utils.xml import validate_xml_data, is_well_formed_xml
from core_main_app.views.common.views import read_xsd_file
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_parser_app.tools.parser.parser import remove_child_element
from core_parser_app.tools.parser.renderer.list import ListRenderer
from xml_utils.xsd_tree.xsd_tree import XSDTree

logger = logging.getLogger(__name__)


# FIXME: delete_branch not deleting all elements
# FIXME: generate element not testing max occurrences


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def start_curate(request):
    """Load forms to start curating.

    Args:
        request:

    Returns:

    """
    try:
        if request.method == "POST":
            return _start_curate_post(request)

        return _start_curate_get(request)
    except CurateAjaxError as exception:
        return HttpResponseBadRequest(escape(str(exception)))
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def generate_choice(request, curate_data_structure_id):
    """Generate a choice branch absent from the form.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        xsd_parser = get_parser(request=request)
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        html_form = xsd_parser.generate_choice_absent(
            element_id, template.content, data_structure=curate_data_structure
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))

    return HttpResponse(html_form)


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def generate_element(request, curate_data_structure_id):
    """Generate an element absent from the form.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        xsd_parser = get_parser(request=request)
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        html_form = xsd_parser.generate_element_absent(
            element_id, template.content, data_structure=curate_data_structure
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))

    return HttpResponse(html_form)


# TODO: need to be reworked
@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def remove_element(request):
    """Remove an element from the form.

    Args:
        request:

    Returns:

    """
    element_id = request.POST["id"]

    # Removing the element from the data structure
    data_structure_element_to_pull = data_structure_element_api.get_by_id(
        element_id, request
    )
    data_structure_element = data_structure_element_to_pull.parent

    # number of children after deletion
    children_number = data_structure_element.children.count() - 1

    data_structure_element = remove_child_element(
        data_structure_element, data_structure_element_to_pull, request
    )

    response = {"code": 0, "html": ""}

    if children_number > data_structure_element.options["min"]:
        return HttpResponse(json.dumps(response))

    # len(schema_element.children) == schema_element.options['min']
    if data_structure_element.options["min"] != 0:
        response["code"] = 1
    else:  # schema_element.options['min'] == 0
        renderer = ListRenderer(data_structure_element, request)
        html_form = renderer.render(True)

        response["code"] = 2
        response["html"] = html_form

    return HttpResponse(json.dumps(response))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def clear_fields(request):
    """Clear fields of the current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # generate form
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        root_element = generate_form(
            template.content,
            data_structure=curate_data_structure,
            request=request,
        )

        # save the root element in the data structure
        curate_data_structure_api.update_data_structure_root(
            curate_data_structure, root_element, request.user
        )

        # renders the form
        xsd_form = render_form(request, root_element)

        return HttpResponse(
            json.dumps({"xsdForm": xsd_form}),
            content_type="application/javascript",
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def cancel_changes(request):
    """Cancel changes of the current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        if curate_data_structure.data is not None:
            # data already saved, reload from data
            xml_data = curate_data_structure.data.xml_content
        elif curate_data_structure.form_string is not None:
            # form already saved, reload from saved form
            xml_data = curate_data_structure.form_string
        else:
            # no saved data, load new form
            xml_data = None

        root_element = generate_root_element(
            request, curate_data_structure, xml_data
        )

        # renders the form
        xsd_form = render_form(request, root_element)

        return HttpResponse(
            json.dumps({"xsdForm": xsd_form}),
            content_type="application/javascript",
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def cancel_form(request):
    """Cancel current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        # unlock from database
        if curate_data_structure.data is not None:
            lock_api.remove_lock_on_object(
                curate_data_structure.data, request.user
            )
        curate_data_structure_api.delete(curate_data_structure, request.user)

        # add success message
        messages.add_message(
            request,
            messages.SUCCESS,
            get_form_label().capitalize() + " deleted with success.",
        )

        return HttpResponse(
            json.dumps({}), content_type="application/javascript"
        )
    except Exception as exception:
        return HttpResponseBadRequest(
            "An unexpected error has occured: %s",
            str(exception).replace('"', "'"),
            content_type="application/javascript",
        )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def save_form(request):
    """Save the current form in data base. Converts it to XML format first.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # generate xml data
        xml_data = render_xml(
            request, curate_data_structure.data_structure_element_root
        )

        # update curate data structure data
        curate_data_structure.form_string = xml_data

        # save data structure
        curate_data_structure_api.upsert(curate_data_structure, request.user)

        # add success message
        message = Message(
            messages.SUCCESS,
            get_form_label().capitalize() + " saved with success.",
        )

        return HttpResponse(
            json.dumps({"message": message.message, "tags": message.tags}),
            content_type="application/json",
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def validate_form(request):
    """Validate data present in the form via XML validation.

    Args:
        request:

    Returns:

    """
    response_dict = {}
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # generate the XML
        xml_data = render_xml(
            request, curate_data_structure.data_structure_element_root
        )

        # build trees
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        xsd_tree = XSDTree.build_tree(template.content)
        xml_tree = XSDTree.build_tree(xml_data)

        # validate XML document
        errors = validate_xml_data(xsd_tree, xml_tree, request=request)

        if errors is not None:
            response_dict["errors"] = errors

    except XMLSyntaxError as xml_syntax_error:
        response_dict[
            "errors"
        ] = "Your XML data is not well formatted. " + str(xml_syntax_error)
    except Exception as exception:
        message = (
            str(exception).replace('"', "'")
            if str(exception) is not None
            else "The current document cannot be validated."
        )
        response_dict["errors"] = message

    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def save_data(request):
    """Save data - delete curate data structure.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # unlock from database
        if curate_data_structure.data is not None:
            lock_api.remove_lock_on_object(
                curate_data_structure.data, request.user
            )

        # generate the XML
        xml_data = render_xml(
            request, curate_data_structure.data_structure_element_root
        )

        if curate_data_structure.data is not None:
            # update existing data
            data = curate_data_structure.data
        else:
            # create new data
            data = Data()
            data.title = curate_data_structure.name
            template = template_api.get_by_id(
                str(curate_data_structure.template.id), request=request
            )
            data.template = template
            data.user_id = str(request.user.id)

        # set content
        data.xml_content = xml_data
        # save data
        data = data_api.upsert(data, request)

        curate_data_structure_api.delete(curate_data_structure, request.user)

        messages.add_message(
            request,
            messages.SUCCESS,
            get_data_label().capitalize() + " saved with success.",
        )
    except Exception as exception:
        return HttpResponseBadRequest(
            str(exception).replace('"', "'"),
            content_type="application/javascript",
        )

    return HttpResponse(
        json.dumps({"data_id": str(data.id)}),
        content_type="application/javascript",
    )


def _start_curate_post(request):
    """Start curate POST request.

    Args:
        request:

    Returns:

    """
    template_id = str(request.POST["hidden_value"])
    selected_option = request.POST["curate_form"]
    user_id = str(request.user.id)

    if selected_option == "new":
        new_form = users_forms.NewForm(request.POST)
        if not new_form.is_valid():
            raise CurateAjaxError(
                "An error occurred during the validation " + get_form_label()
            )
        name = new_form.data["document_name"]
        curate_data_structure = CurateDataStructure(
            user=user_id,
            template=template_api.get_by_id(template_id, request=request),
            name=name,
        )
        curate_data_structure_api.upsert(curate_data_structure, request.user)
        url = _get_reverse_url(new_form, curate_data_structure.id)
    elif selected_option == "upload":
        upload_form = users_forms.UploadForm(request.POST, request.FILES)
        if not upload_form.is_valid():
            raise CurateAjaxError(
                f"An error occurred during the file upload: {upload_form.errors.as_text()}"
            )
        xml_file = request.FILES["file"]
        xml_data = read_xsd_file(xml_file)
        well_formed = is_well_formed_xml(xml_data)
        name = xml_file.name
        if not well_formed:
            raise CurateAjaxError(
                "An error occurred during the file upload: the file is "
                "not well formed XML"
            )
        if (
            "direct_upload" in upload_form.data
            and upload_form.data["direct_upload"]
        ):
            # get template
            template = template_api.get_by_id(
                template_id,
                request=request,
            )
            # create data
            data = Data(
                title=name, template=template, user_id=str(request.user.id)
            )
            # set content
            data.xml_content = xml_data
            # save data
            data_api.upsert(data, request)
            messages.add_message(
                request,
                messages.SUCCESS,
                get_data_label().capitalize() + " saved with success.",
            )
            url = reverse("core_main_app_data_detail") + f"?id={str(data.id)}"
        else:
            curate_data_structure = CurateDataStructure(
                user=user_id,
                template=template_api.get_by_id(template_id, request=request),
                name=name,
                form_string=xml_data,
            )
            curate_data_structure_api.upsert(
                curate_data_structure, request.user
            )
            url = _get_reverse_url(upload_form, curate_data_structure.id)
    else:
        open_form = users_forms.OpenForm(request.POST)
        curate_data_structure = curate_data_structure_api.get_by_id(
            open_form.data["forms"], request.user
        )
        url = _get_reverse_url(open_form, curate_data_structure.id)
    return HttpResponse(url)


def _get_reverse_url(form, curate_data_structure_id):
    """get reverse url.

    Args:
        form:
        curate_data_structure_id

    Returns:
        url:
    """
    if "text_editor" in form.data:
        url = (
            reverse("core_curate_app_xml_text_editor_view")
            + f"?id={str(curate_data_structure_id)}"
        )
    else:
        url = reverse(
            "core_curate_enter_data", args=(curate_data_structure_id,)
        )
    return url


def _start_curate_get(request):
    """Start curate GET request.

    Args:
        request:

    Returns:

    """
    try:
        context_params = dict()
        template_id = request.GET["template_id"]
        template = loader.get_template(
            "core_curate_app/user/curate_start.html"
        )

        open_form = users_forms.OpenForm(
            forms=curate_data_structure_api.get_all_by_user_id_and_template_id_with_no_data(
                request.user.id, template_id
            )
        )
        new_form = users_forms.NewForm()
        upload_form = users_forms.UploadForm()
        hidden_form = users_forms.HiddenFieldsForm(hidden_value=template_id)
        context_params["new_form"] = new_form
        context_params["open_form"] = open_form
        context_params["upload_form"] = upload_form
        context_params["hidden_form"] = hidden_form
        context_params["request"] = request
        context = {}

        context.update(context_params)
        return HttpResponse(
            json.dumps({"template": template.render(context)}),
            content_type="application/javascript",
        )
    except Exception as exception:
        logger.error(str(exception))
        raise exceptions.CurateAjaxError(
            "Error occurred during the " + get_form_label() + " display."
        )



""" Custom admin site for the Curate Data Structure model
"""
from django.contrib import admin


class CustomCurateDataStructureAdmin(admin.ModelAdmin):
    """CustomCurateDataStructureAdmin"""

    readonly_fields = [
        "template",
        "data_structure_element_root",
        "form_string",
        "data",
    ]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Curate Data Structures"""
        return False



""" Curate data Structure api
"""
from core_main_app.access_control.api import has_perm_administration
from core_main_app.access_control.decorators import access_control
from core_curate_app.access_control.api import (
    can_read,
    can_write,
    can_change_owner,
)
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)


@access_control(can_write)
def upsert(curate_data_structure, user):
    """Save or update the Curate Data Structure

    Args:
        curate_data_structure:
        user:

    Returns:

    """
    curate_data_structure.save_object()
    return curate_data_structure


def get_none():
    """Returns None object, used by forms

    Returns:

    """
    return CurateDataStructure.get_none()


@access_control(has_perm_administration)
def get_all(user):
    """Returns all curate data structure api

    Returns:

    """
    return CurateDataStructure.get_all()


@access_control(can_read)
def get_by_id(curate_data_structure_id, user):
    """Returns the curate data structure with the given id

    Args:
        curate_data_structure_id:
        user:

    Returns:

    """
    return CurateDataStructure.get_by_id(curate_data_structure_id)


def get_all_by_user_id_and_template_id(user_id, template_id):
    """Returns object with the given user id and template id

    Args:
        user_id:
        template_id:

    Returns:

    """
    return CurateDataStructure.get_all_by_user_id_and_template_id(
        user_id, template_id
    )


def get_by_user_id_and_template_id_and_name(user_id, template_id, name):
    """Returns object with the given user id and template id and name

    Args:
        user_id:
        template_id:
        name:

    Returns:

    """
    return CurateDataStructure.get_by_user_id_and_template_id_and_name(
        user_id, template_id, name
    )


@access_control(can_write)
def delete(curate_data_structure, user):
    """Deletes the curate data structure and the element associated

    Args:
        curate_data_structure:
        user:
    """
    curate_data_structure.delete()


def get_all_by_user_id_with_no_data(user_id):
    """Returns all the curate date structure of the user, with no data.

    Args:
        user_id:
    Returns:
    """
    return CurateDataStructure.get_all_by_user_id_with_no_data(user_id)


@access_control(has_perm_administration)
def get_all_except_user_id_with_no_data(user_id, user):
    """Returns all the curate date structure except the one of the user, with no data.

    Args:
        user_id:
    Returns:
    """
    return CurateDataStructure.get_all_except_user_id_with_no_data(user_id)


def get_all_by_user_id_and_template_id_with_no_data(user_id, template_id):
    """Return all the curate data structure by user and template, with no link to a data

    Args:
        user_id:
        template_id:

    Returns:

    """
    return CurateDataStructure.get_all_by_user_id_and_template_id_with_no_data(
        user_id, template_id
    )


@access_control(has_perm_administration)
def get_all_with_no_data(user):
    """Returns all curate data structure api with no link to a data.

    Returns:

    """
    return CurateDataStructure.get_all_with_no_data()


@access_control(can_read)
def get_by_data_id(data_id, user):
    """Return the curate data structure with the given data id

    Args:
        data_id:

    Returns:

    """
    return CurateDataStructure.get_by_data_id(data_id)


@access_control(can_read)
def get_all_by_user(user):
    """Get all curate data that belong to user.

    Args:
        user: User

    Returns:

    """
    return CurateDataStructure.get_all_by_user(user.id)


@access_control(can_write)
def update_data_structure_root(curate_data_structure, root_element, user):
    """Update the data structure with a root element.

    Args:
        curate_data_structure:
        root_element:

    Returns:

    """
    # Delete data structure elements
    curate_data_structure.delete_data_structure_elements_from_root()

    # set the root element in the data structure
    curate_data_structure.data_structure_element_root = root_element

    # save the data structure
    upsert(curate_data_structure, user)

    return curate_data_structure


@access_control(can_read)
def get_by_data_structure_element_root_id(data_structure_element_root, user):
    """Return the curate data structure with the given data structure element root id

    Args:
        data_structure_element_root:
        user:

    Returns:

    """
    return CurateDataStructure.get_by_data_structure_element_root(
        data_structure_element_root
    )


@access_control(can_change_owner)
def change_owner(curate_data_structure, new_user, user):
    """Change curate data structure's owner.

    Args:
        curate_data_structure:
        user:
        new_user:

    Returns:
    """
    curate_data_structure.user = str(new_user.id)
    curate_data_structure.save_object()



""" Url router for the curate application
"""
from django.conf.urls import include
from django.urls import re_path, reverse_lazy
from core_main_app.utils.decorators import permission_required
from core_curate_app.permissions import rights
import core_curate_app.views.user.ajax as user_ajax
import core_curate_app.views.user.views as user_views
from core_curate_app.views.common import views as common_views


urlpatterns = [
    re_path(r"^$", user_views.index, name="core_curate_index"),
    re_path(
        r"^start_curate$", user_ajax.start_curate, name="core_curate_start"
    ),
    re_path(
        r"^enter-data/(?P<curate_data_structure_id>\w+)$",
        user_views.EnterDataView.as_view(),
        name="core_curate_enter_data",
    ),
    # FIXME: url to allow reopening a form with unsaved changes
    #  (may be temporary until curate workflow redesign)
    re_path(
        r"^enter-data/(?P<curate_data_structure_id>\w+)/(?P<reload_unsaved_changes>\w+)$",
        user_views.EnterDataView.as_view(),
        name="core_curate_enter_data",
    ),
    re_path(
        r"^view-data/(?P<curate_data_structure_id>\w+)$",
        user_views.ViewDataView.as_view(),
        name="core_curate_view_data",
    ),
    re_path(
        r"^download-xml/(?P<curate_data_structure_id>\w+)$",
        user_views.download_current_xml,
        name="core_curate_download_xml",
    ),
    re_path(
        r"^download-xsd/(?P<curate_data_structure_id>\w+)$",
        user_views.download_xsd,
        name="core_curate_download_xsd",
    ),
    re_path(
        r"^generate-choice/(?P<curate_data_structure_id>\w+)$",
        user_ajax.generate_choice,
        name="core_curate_generate_choice",
    ),
    re_path(
        r"^generate-element/(?P<curate_data_structure_id>\w+)$",
        user_ajax.generate_element,
        name="core_curate_generate_element",
    ),
    re_path(
        r"^remove-element$",
        user_ajax.remove_element,
        name="core_curate_remove_element",
    ),
    re_path(
        r"^clear-fields$",
        user_ajax.clear_fields,
        name="core_curate_clear_fields",
    ),
    re_path(
        r"^cancel-changes$",
        user_ajax.cancel_changes,
        name="core_curate_cancel_changes",
    ),
    re_path(
        r"^cancel-form$", user_ajax.cancel_form, name="core_curate_cancel_form"
    ),
    re_path(
        r"^xml-editor/form",
        permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_DATA_STRUCTURE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )(common_views.DraftContentEditor.as_view()),
        name="core_curate_app_xml_text_editor_view",
    ),
    re_path(r"^save-form$", user_ajax.save_form, name="core_curate_save_form"),
    re_path(r"^save-data$", user_ajax.save_data, name="core_curate_save_data"),
    re_path(
        r"^validate-form$",
        user_ajax.validate_form,
        name="core_curate_validate_form",
    ),
    re_path(
        r"^view-form/(?P<curate_data_structure_id>\w+)$",
        permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )(common_views.FormView.as_view()),
        name="core_curate_view_form",
    ),
    re_path(r"^rest/", include("core_curate_app.rest.urls")),
]



"""Curate models
"""

from django.db import models

from core_main_app.permissions.utils import get_formatted_name
from core_curate_app.permissions import rights


class Curate(models.Model):
    """Curate"""

    class Meta:
        """Meta"""

        verbose_name = "core_curate_app"
        default_permissions = ()
        permissions = (
            (rights.CURATE_ACCESS, get_formatted_name(rights.CURATE_ACCESS)),
            (
                rights.CURATE_VIEW_DATA_SAVE_REPO,
                get_formatted_name(rights.CURATE_VIEW_DATA_SAVE_REPO),
            ),
            (
                rights.CURATE_DATA_STRUCTURE_ACCESS,
                get_formatted_name(rights.CURATE_DATA_STRUCTURE_ACCESS),
            ),
        )



""" Apps file for setting core package when app is ready
"""
import sys

from django.apps import AppConfig


class CurateAppConfig(AppConfig):
    """Core application settings."""

    name = "core_curate_app"
    verbose_name = "Core Curate App"

    def ready(self):
        """Run when the app is ready.

        Returns:

        """
        if "migrate" not in sys.argv:
            from core_curate_app.permissions import discover

            discover.init_permissions()



""" Test forms
"""
from unittest.case import TestCase
from unittest.mock import MagicMock

from django.core.exceptions import ValidationError

from core_curate_app.views.user.forms import _file_size_validator
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE


class TestFileSizeValidator(TestCase):
    """Test _file_size_validator"""

    def test_file_size_validator_raises_error_when_file_too_large(
        self,
    ):
        """test_file_size_validator_raises_error_when_file_too_large

        Returns:

        """
        mock_field = MagicMock()
        mock_field.size = MAX_DOCUMENT_EDITING_SIZE + 1
        with self.assertRaises(ValidationError):
            _file_size_validator(mock_field)

    def test_file_size_validator_raises_ok_when_file_size_under_limit(
        self,
    ):
        """test_file_size_validator_raises_ok_when_file_size_under_limit

        Returns:

        """
        mock_field = MagicMock()
        mock_field.size = MAX_DOCUMENT_EDITING_SIZE - 1
        _file_size_validator(mock_field)



"""
    Common views
"""
import json

from django.contrib import messages
from django.http import (
    HttpResponseBadRequest,
    HttpResponseForbidden,
    HttpResponse,
)
from django.urls import reverse
from django.utils.html import escape as html_escape
from django.utils.translation import ugettext as _

from core_curate_app.components.curate_data_structure import (
    api as data_structure_api,
)
from core_curate_app.views.user.views import _get_curate_data_structure_by_id
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import (
    DoesNotExist,
    DocumentEditingSizeError,
)
from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.template import api as template_api
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE
from core_main_app.utils import file as main_file_utils
from core_main_app.utils.labels import get_data_label
from core_main_app.utils.rendering import render
from core_main_app.views.common.views import CommonView, XmlEditor


class FormView(CommonView):
    """
    Form View
    """

    template = "core_curate_app/user/detail.html"

    def get(self, request, *args, **kwargs):
        """Get the web page of a given form

        Args:
            request:

        Returns:
        """
        try:
            # get data structure
            curate_data_structure = _get_curate_data_structure_by_id(
                kwargs["curate_data_structure_id"], request
            )

            # TODO: fix with the rework on the curate workflow
            if curate_data_structure.form_string is None:
                raise Exception(
                    "The "
                    + _("form_label")
                    + " was not saved. We can't display the correct data."
                )

            # Set the assets
            assets = {
                "js": [
                    {
                        "path": "core_main_app/common/js/XMLTree.js",
                        "is_raw": False,
                    },
                    {
                        "path": "core_main_app/user/js/data/detail.js",
                        "is_raw": False,
                    },
                ],
                "css": ["core_main_app/common/css/XMLTree.css"],
            }

            # Set the context
            context = {
                "data_structure": curate_data_structure,
                "page_title": "View Form",
            }

            return self.common_render(
                request, self.template, assets=assets, context=context
            )
        except Exception as exception:
            assets = {
                "js": [
                    {
                        "path": "core_main_app/common/js/backtoprevious.js",
                        "is_raw": True,
                    }
                ]
            }
            template = "core_main_app/common/commons/error.html"
            if self.administration:
                template = (
                    "core_main_app/admin/commons/errors/errors_wrapper.html"
                )

            return self.common_render(
                request,
                template,
                context={"error": str(exception), "page_title": "Error"},
                assets=assets,
            )


class DraftContentEditor(XmlEditor):
    """Draft Content Editor View"""

    def get(self, request):
        """get

        Args:
            request

        Returns:
        """

        try:

            data_structure = data_structure_api.get_by_id(
                request.GET["id"], request.user
            )
            if (
                main_file_utils.get_byte_size_from_string(
                    data_structure.form_string
                )
                > MAX_DOCUMENT_EDITING_SIZE
            ):
                raise DocumentEditingSizeError(
                    "The file is too large (MAX_DOCUMENT_EDITING_SIZE)."
                )
            context = self.get_context(
                data_structure, data_structure.name, data_structure.form_string
            )
            assets = self._get_assets()
            modals = ["core_main_app/common/modals/create_data_modal.html"]
            return render(
                request,
                self.template,
                assets=assets,
                context=context,
                modals=modals,
            )
        except AccessControlError:
            error_message = "Access Forbidden"
            status_code = 403
        except DoesNotExist:
            error_message = "Form not found"
            status_code = 404
        except Exception as e:
            error_message = str(e)
            status_code = 400

        return render(
            request,
            "core_main_app/common/commons/error.html",
            assets={
                "js": [
                    {
                        "path": "core_main_app/user/js/data/detail.js",
                        "is_raw": False,
                    }
                ]
            },
            context={
                "error": error_message,
                "status_code": status_code,
            },
        )

    def save(self, *args, **kwargs):
        """Save xml content

        Args:
            args:
            kwargs:

        Returns:

        """
        try:
            content = self.request.POST["content"].strip()
            data_structure_id = self.request.POST["document_id"]
            data_structure = data_structure_api.get_by_id(
                data_structure_id, self.request.user
            )

            # create new data
            data = Data()
            data.title = data_structure.name
            template = template_api.get_by_id(
                str(data_structure.template.id), request=self.request
            )
            data.template = template
            data.user_id = str(self.request.user.id)

            # set content
            data.xml_content = content
            # save data
            data = data_api.upsert(data, self.request)

            data_structure_api.delete(data_structure, self.request.user)
            messages.add_message(
                self.request,
                messages.SUCCESS,
                get_data_label() + " saved with success.",
            )
            return HttpResponse(
                json.dumps({"url": reverse(self.save_redirect)}),
                "application/javascript",
            )
        except AccessControlError as ace:
            return HttpResponseForbidden(html_escape(str(ace)))
        except DoesNotExist as dne:
            return HttpResponseBadRequest(html_escape(str(dne)))
        except Exception as e:
            return HttpResponseBadRequest(html_escape(str(e)))

    def _get_assets(self):
        assets = super()._get_assets()

        # add js relatives to the xml editor
        assets["js"].append(
            {
                "path": "core_curate_app/user/js/text_editor.raw.js",
                "is_raw": True,
            },
        )
        return assets



""" Fixtures files for Data Structure
"""
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)
from django.core.files.uploadedfile import SimpleUploadedFile
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)


class DataStructureFixtures(FixtureInterface):
    """Data structure fixtures"""

    data_structure_1 = None
    data_structure_2 = None
    data_structure_3 = None
    data = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title",
        )
        self.data.save()

        self.data_structure_1 = CurateDataStructure(
            user="1",
            template=self.template,
            name="data_structure_1",
            data=self.data,
        )
        self.data_structure_1.save()

        self.data_structure_2 = CurateDataStructure(
            user="1", template=self.template, name="data_structure_2"
        )
        self.data_structure_2.save()

        self.data_structure_3 = CurateDataStructure(
            user="2", template=self.template, name="data_structure_3"
        )
        self.data_structure_3.save()

        self.data_collection = [
            self.data_structure_1,
            self.data_structure_2,
            self.data_structure_3,
            self.data,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.file = SimpleUploadedFile(
            "user1_template.xsd", xsd.encode("utf-8")
        )
        self.template.user = "1"
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()



""" Test views
"""
from unittest.case import TestCase
from unittest.mock import patch, MagicMock

from django.http import HttpResponseRedirect
from django.test import RequestFactory
from django.urls import reverse

from core_curate_app.views.common.views import DraftContentEditor
from core_curate_app.views.user.views import EnterDataView
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_parser_app.tools.parser.exceptions import ParserError


class TestEnterDataView(TestCase):
    """Test EnterDataView"""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user1.has_perm = MagicMock()
        self.user1.has_perm.return_value = True

    @patch("core_curate_app.views.user.views.EnterDataView.build_context")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_view_get_returns_error_if_parser_error_occurs(
        self, mock_ds_get_by_id, mock_build_context
    ):
        """test_enter_data_view_get_returns_error_if_parser_error_occurs


        Returns:


        """
        # Arrange
        mock_build_context.side_effect = ParserError("error")
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds.data = None
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        # Act
        response = EnterDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertTrue(
            response.url.startswith(
                reverse("core_curate_app_xml_text_editor_view")
            )
        )


class TestDraftContentEditor(TestCase):
    """Test Draft Content Editor View"""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")

    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    @patch("core_main_app.utils.file.get_byte_size_from_string")
    def test_xml_content_too_big_returns_error(
        self, mock_get_byte_size, mock_ds_get_by_id
    ):
        """test_user_save_xml_content_returns_error


        Returns:


        """
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds_get_by_id.return_value = mock_ds
        mock_get_byte_size.return_value = MAX_DOCUMENT_EDITING_SIZE + 1
        request = self.factory.get("core_main_app_xml_text_editor_view")
        request.GET = {"id": 1}
        request.user = self.user1
        response = DraftContentEditor.as_view()(request)
        self.assertTrue(
            "MAX_DOCUMENT_EDITING_SIZE" in response.content.decode()
        )



""" Rights for core curate app
"""

CURATE_CONTENT_TYPE = "core_curate_app"
CURATE_ACCESS = "access_curate"
CURATE_VIEW_DATA_SAVE_REPO = "view_data_save_repo"
CURATE_DATA_STRUCTURE_ACCESS = "access_curate_data_structure"



"""
Url router for the administration site
"""
from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path

from core_main_app.admin import core_admin_site
from core_curate_app.components.curate_data_structure.admin_site import (
    CustomCurateDataStructureAdmin,
)
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.views.common import views as common_views

admin_urls = [
    re_path(
        r"^view-form/(?P<curate_data_structure_id>\w+)$",
        staff_member_required(
            common_views.FormView.as_view(
                administration=True,
                template="core_curate_app/admin/view_curate_data_structure.html",
            )
        ),
        name="core_curate_view_form",
    )
]

admin.site.register(CurateDataStructure, CustomCurateDataStructureAdmin)
urls = core_admin_site.get_urls()
core_admin_site.get_urls = lambda: admin_urls + urls



""" Add Curate in main menu
"""

from django.urls import reverse
from menu import Menu, MenuItem

from core_curate_app.settings import CURATE_MENU_NAME

Menu.add_item("main", MenuItem(CURATE_MENU_NAME, reverse("core_curate_index")))



""" Unit Test Curate Data Structure
"""
from unittest.case import TestCase

from unittest.mock import patch

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)


class TestCurateDataStructureGetById(TestCase):
    """
    Test Curate Data Structure Get By Id
    """

    @patch.object(CurateDataStructure, "get_by_id")
    def test_curate_data_structure_get_by_id_raises_does_not_exist_error_if_not_found(
        self, mock_get
    ):
        """
        test_curate_data_structure_get_by_id_raises_does_not_exist_error_if_not_found

        Returns:

        """
        # Arrange
        mock_get.side_effect = exceptions.DoesNotExist("")
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            curate_data_structure_api.get_by_id(1, mock_user)

    def test_data_structure_get_by_id_raises_does_not_exist_error_if_not_found(
        self,
    ):
        """
        test_data_structure_get_by_id_raises_does_not_exist_error_if_not_found

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            curate_data_structure_api.get_by_id(1, mock_user)

    @patch.object(CurateDataStructure, "get_by_id")
    def test_curate_data_structure_get_by_id_return_data_if_found(
        self, mock_get
    ):
        """
        test_curate_data_structure_get_by_id_return_data_if_found

        Returns:

        """
        # Arrange
        mock_data_structure = CurateDataStructure(
            user="1", template=Template(), name="name"
        )
        mock_get.return_value = mock_data_structure
        mock_user = create_mock_user("1")
        # Act
        result = curate_data_structure_api.get_by_id(1, mock_user)
        # Assert
        self.assertIsInstance(result, CurateDataStructure)


class TestCurateDataStructureUpsert(TestCase):
    """
    Test Curate Data Structure Upsert
    """

    @patch.object(CurateDataStructure, "save_object")
    def test_curate_data_structure_upsert_return_data_structure_element(
        self, mock_save
    ):
        """
        test_curate_data_structure_upsert_return_data_structure_element

        Returns:

        """
        # Arrange
        mock_data_structure = CurateDataStructure(
            user="1", template=Template(), name="name"
        )
        mock_save.return_value = mock_data_structure
        mock_user = create_mock_user("1")
        # Act
        result = curate_data_structure_api.upsert(
            mock_data_structure, mock_user
        )
        # Assert
        self.assertIsInstance(result, CurateDataStructure)


class TestCurateDataStructureGetAll(TestCase):
    """
    Test Curate Data Structure Get All
    """

    @patch.object(CurateDataStructure, "get_all")
    def test_curate_data_get_all_return_collection_of_curate_data(
        self, mock_list
    ):
        """
        test_curate_data_get_all_return_collection_of_curate_data

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]
        # Act
        result = curate_data_structure_api.get_all(
            create_mock_user("1", is_staff=True, is_superuser=True)
        )
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


class TestCurateDataStructureGetByUserIdAndTemplateId(TestCase):
    """
    Test Curate Data Structure Get By User Id And Template Id
    """

    @patch.object(CurateDataStructure, "get_all_by_user_id_and_template_id")
    def test_curate_data_structure_get_all_by_user_and_template_return_collection(
        self, mock_list
    ):
        """
        test_curate_data_structure_get_all_by_user_and_template_return_collection

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]

        # Act
        result = curate_data_structure_api.get_all_by_user_id_and_template_id(
            1, 1
        )
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


class TestCurateDataStructureGetByUserIdAndTemplateIdAndName(TestCase):
    """
    Test Curate Data Structure Get By User Id And Template Id And Name
    """

    @patch.object(
        CurateDataStructure, "get_by_user_id_and_template_id_and_name"
    )
    def test_curate_data_structure_get_by_user_and_template_and_name_return_curate_data_structure(
        self, mock_get
    ):
        """
        test_curate_data_structure_get_by_user_and_template_and_name_return_curate_data_structure

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_get.return_value = mock_data_1
        # Act
        result = (
            curate_data_structure_api.get_by_user_id_and_template_id_and_name(
                1, 1, "name_title_1"
            )
        )
        # Assert
        self.assertIsInstance(result, CurateDataStructure)

    @patch.object(
        CurateDataStructure, "get_by_user_id_and_template_id_and_name"
    )
    def test_curate_data_structure_get_by_user_and_template_and_name_raises_error_if_not_found(
        self, mock_get
    ):
        """
        test_curate_data_structure_get_by_user_and_template_and_name_raises_error_if_not_found

        Returns:

        """
        # Arrange
        mock_get.side_effect = exceptions.DoesNotExist("")
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            curate_data_structure_api.get_by_user_id_and_template_id_and_name(
                1, 1, "name_title_1"
            )

    def test_data_structure_get_by_user_and_template_and_name_raises_error_if_not_found(
        self,
    ):
        """
        test_data_structure_get_by_user_and_template_and_name_raises_error_if_not_found

        Returns:

        """

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            curate_data_structure_api.get_by_user_id_and_template_id_and_name(
                1, 1, "name_title_1"
            )


class TestCurateDataStructureGetAllByUserIdWithNoData(TestCase):
    """
    Test Curate Data Structure Get All By User Id With No Data
    """

    @patch.object(CurateDataStructure, "get_all_by_user_id_with_no_data")
    def test_curate_data_structure_get_all_by_user_id_with_no_data_return_curate_data_structure(
        self, mock_list
    ):
        """
        test_curate_data_structure_get_all_by_user_id_with_no_data_return_curate_data_structure

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]
        # Act
        result = curate_data_structure_api.get_all_by_user_id_with_no_data(1)
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


class TestCurateDataStructureGetAllExceptUserIdWithNoData(TestCase):
    """
    Test Curate Data Structure Get All Except User Id With No Data
    """

    @patch.object(CurateDataStructure, "get_all_except_user_id_with_no_data")
    def test_curate_data_structure_get_all_except_user_id_with_no_data_return_curate_data_structure(
        self, mock_list
    ):
        """
        test_curate_data_structure_get_all_except_user_id_with_no_data_return_curate_data_structure

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        result = curate_data_structure_api.get_all_except_user_id_with_no_data(
            2, mock_user
        )
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


class TestCurateDataStructureGetAllByUserIdAndTemplateIdWithNoData(TestCase):
    """
    Test Curate Data Structure Get All By User Id And Template Id With No Data
    """

    @patch.object(
        CurateDataStructure, "get_all_by_user_id_and_template_id_with_no_data"
    )
    def test_curate_data_structure_get_all_by_user_id_and_template_id_with_no_data_returns_data(
        self, mock_list
    ):
        """
        test_curate_data_structure_get_all_by_user_id_and_template_id_with_no_data_returns_data

        Returns:

        """

        # Arrange
        template = _get_template()
        mock_data_1 = CurateDataStructure(
            user="1", template=template, name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=template, name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]
        # Act
        result = curate_data_structure_api.get_all_by_user_id_and_template_id_with_no_data(
            "1", template.id
        )
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


class TestCurateDataStructureGetByUserId(TestCase):
    """
    Test Curate Data Structure Get By User Id
    """

    @patch.object(CurateDataStructure, "get_all_by_user")
    def test_curate_data_structure_get_all_by_user_return_collection_of_curate_data_structure(
        self, mock_list
    ):
        """
        test_curate_data_structure_get_all_by_user_return_collection_of_curate_data_structure

        Returns:

        """
        # Arrange
        mock_data_1 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_1"
        )
        mock_data_2 = CurateDataStructure(
            user="1", template=_get_template(), name="name_title_2"
        )
        mock_list.return_value = [mock_data_1, mock_data_2]
        # Act
        result = curate_data_structure_api.get_all_by_user(create_mock_user(1))
        # Assert
        self.assertTrue(
            all(isinstance(item, CurateDataStructure) for item in result)
        )


def _get_template():
    template = Template()
    template.id_field = 1
    xsd = (
        '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
        '<xs:element name="tag"></xs:element></xs:schema>'
    )
    template.content = xsd
    return template
